import os
import sys
import shutil
import traceback
from optparse import OptionParser
import SSSDConfig


# Older versions of SSSD (1.5 and earlier) would take a debug_level
# value set in the [sssd] section as authoritative for all other
# sections where not explicitly overridden. We changed this so that
# all sections need to set it if they want debug logs set.
# This script can be run to make the new version continue to produce
# the same logs as the old versions did, by explicitly adding
# debug_level to all domains and services that did not have it set
# already.

def parse_options():
    parser = OptionParser()
    parser.add_option("", "--no-backup", action="store_false",
                      dest="backup", default=True,
                      help="""Do not provide backup file after conversion.
The script copies the original file with the suffix .bak.<timestamp>
by default""")
    parser.add_option("-v", "--verbose", action="store_true",
                      dest="verbose", default=False,
                      help="Be verbose")
    (options, args) = parser.parse_args()
    if len(args) > 0:
        print >>sys.stderr, "Stray arguments: %s" % ' '.join([a for a in args])
        return None

    return options

def verbose(msg, verbosity):
    if verbosity:
        print msg

def main():
    options = parse_options()
    if not options:
        print >> sys.stderr, "Cannot parse options"
        return 1

    # Import the current config file
    try:
        sssdconfig = SSSDConfig.SSSDConfig()
        sssdconfig.import_config()

    except Exception, e:
        print "Error: %s" % e
        verbose(traceback.format_exc(), options.verbose)
        return 2

    # Check the [sssd] section for debug_level
    sssd_service = sssdconfig.get_service('sssd')

    if not 'debug_level' in sssd_service.options.keys():
        # Nothing to do, just return success
        verbose("No changes required, no backup necessary",
                options.verbose)
        return 0

    debug_level = sssd_service.options['debug_level']
    verbose("Setting all sections to debug_level = %d" % debug_level,
            options.verbose)

    # Loop through services
    for service in sssdconfig.list_services():
        svc = sssdconfig.get_service(service)
        if not 'debug_level' in svc.options.keys():
            # Not explicitly set, so add it
            svc.set_option('debug_level', debug_level)
            sssdconfig.save_service(svc)

    # Loop through domains (active AND inactive)
    for domain in sssdconfig.list_domains():
        dom = sssdconfig.get_domain(domain)
        if not 'debug_level' in dom.options.keys():
            # Not explicitly set, so add it
            dom.set_option('debug_level', debug_level)
            sssdconfig.save_domain(dom)

    # Save the original file
    if options.backup:
        import datetime
        currenttime = datetime.datetime.utcnow()
        newfile = "/etc/sssd/sssd.conf.bak.%s" % currenttime.isoformat()
        verbose("Backing up existing configuration to %s" % newfile,
                options.verbose)
        shutil.copy2("/etc/sssd/sssd.conf", newfile)

    # Save the changes
    sssdconfig.write()

if __name__ == "__main__":
    ret = main()
    sys.exit(ret)
else:
    raise ImportError



#coding=utf-8

#  SSSD
#
#  upgrade_config.py
#
#  Copyright (C) Jakub Hrozek <jhrozek@redhat.com>        2009
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import shutil
import traceback
from optparse import OptionParser

from ipachangeconf import openLocked
from ipachangeconf import SSSDChangeConf

class SSSDConfigFile(SSSDChangeConf):
    def __init__(self, filename):
        SSSDChangeConf.__init__(self)
        self.filename = filename

        f = openLocked(self.filename, 0600, False)
        self.opts = self.parse(f)
        f.close()

    def _backup_file(self, file_name):
        " Copy the file we operate on to a backup location "
        shutil.copy(file_name, file_name + self.backup_suffix)
        # make sure we don't leak data, force permissions on the backup
        os.chmod(file_name + self.backup_suffix, 0600)

    def get_version(self):
        ver = self.get_option_index('sssd', 'config_file_version')[1]
        if not ver:
            return 1
        try:
            return int(ver['value'])
        except ValueError:
            raise SyntaxError, 'config_file_version not an integer'

    def rename_opts(self, parent_name, rename_kw, type='option'):
        for new_name, old_name in rename_kw.items():
            index, item = self.get_option_index(parent_name, old_name, type)
            if item:
                item['name'] = new_name

    def _add_dns_domain_name(self, domain):
        id_provider = self.findOpts(domain['value'], 'option', 'id_provider')[1]
        dns_domain_name = { 'type' : 'option',
                            'name' : 'dns_discovery_domain',
                            'value' : domain['name'].lstrip('domain/') }
        if id_provider['value'] == 'ldap':
            server = self.findOpts(domain['value'], 'option', 'ldap_uri')[1]
            if not server or "__srv__" in server['value']:
                domain['value'].insert(0, dns_domain_name)
                return
        elif id_provider['value'] == 'ipa':
            server = self.findOpts(domain['value'], 'option', 'ipa_server')[1]
            if not server or "__srv__" in server['value']:
                domain['value'].insert(0, dns_domain_name)
                return

        auth_provider = self.findOpts(domain['value'], 'option', 'auth_provider')[1]
        if auth_provider and auth_provider['value'] == 'krb5':
            server = self.findOpts(domain['value'], 'option', 'krb5_server')[1]
            if not server or "__srv__" in server['value']:
                domain['value'].insert(0, dns_domain_name)

    def _do_v2_changes(self):
        # remove Data Provider
        srvlist = self.get_option_index('sssd', 'services')[1]
        if srvlist:
            services = [ srv.strip() for srv in srvlist['value'].split(',') ]
            if 'dp' in services:
                services.remove('dp')
            srvlist['value'] = ", ".join([srv for srv in services])
        self.delete_option('section', 'dp')

        for domain in [ s for s in self.sections() if s['name'].startswith("domain/") ]:
            # remove magic_private_groups from all domains
            self.delete_option_subtree(domain['value'], 'option', 'magic_private_groups')
            # check if we need to add dns_domain
            self._add_dns_domain_name(domain)

    def _update_option(self, to_section_name, from_section_name, opts):
        to_section = [ s for s in self.sections() if s['name'].strip() == to_section_name ]
        from_section = [ s for s in self.sections() if s['name'].strip() == from_section_name ]

        if len(to_section) > 0 and len(from_section) > 0:
            vals = to_section[0]['value']
            for o in [one_opt for one_opt in from_section[0]['value'] if one_opt['name'] in opts]:
                updated = False
                for v in vals:
                    if v['type'] == 'empty':
                        continue
                    # if already in list, just update
                    if o['name'] == v['name']:
                        o['value'] = v['value']
                        updated = True
                # not in list, add there
                if not updated:
                    vals.insert(0, { 'name' : o['name'], 'type' : o['type'], 'value' : o['value'] })

    def _migrate_enumerate(self, domain):
        " Enumerate was special as it turned into bool from (0,1,2,3) enum "
        enum = self.findOpts(domain, 'option', 'enumerate')[1]
        if enum:
            if enum['value'].upper() not in ['TRUE', 'FALSE']:
                try:
                    enum['value'] = int(enum['value'])
                except ValueError:
                    raise ValueError('Cannot convert value %s in domain %s' % (enum['value'], domain['name']))

                if enum['value'] == 0:
                    enum['value'] = 'FALSE'
                elif enum['value'] > 0:
                    enum['value'] = 'TRUE'
                else:
                    raise ValueError('Cannot convert value %s in domain %s' % (enum['value'], domain['name']))

    def _migrate_domain(self, domain):
        # rename the section
        domain['name'] = domain['name'].strip().replace('domains', 'domain')

        # Generic options - new:old
        generic_kw = { 'min_id' : 'minId',
                       'max_id': 'maxId',
                       'timeout': 'timeout',
                       'magic_private_groups' : 'magicPrivateGroups',
                       'cache_credentials' : 'cache-credentials',
                       'id_provider' : 'provider',
                       'auth_provider' : 'auth-module',
                       'access_provider' : 'access-module',
                       'chpass_provider' : 'chpass-module',
                       'use_fully_qualified_names' : 'useFullyQualifiedNames',
                       'store_legacy_passwords' : 'store-legacy-passwords',
                      }
        # Proxy options
        proxy_kw = { 'proxy_pam_target' : 'pam-target',
                     'proxy_lib_name'   : 'libName',
                   }
        # LDAP options - new:old
        ldap_kw = { 'ldap_uri' : 'ldapUri',
                    'ldap_schema' : 'ldapSchema',
                    'ldap_default_bind_dn' : 'defaultBindDn',
                    'ldap_default_authtok_type' : 'defaultAuthtokType',
                    'ldap_default_authtok' : 'defaultAuthtok',
                    'ldap_user_search_base' : 'userSearchBase',
                    'ldap_user_search_scope' : 'userSearchScope',
                    'ldap_user_search_filter' : 'userSearchFilter',
                    'ldap_user_object_class' : 'userObjectClass',
                    'ldap_user_name' : 'userName',
                    'ldap_user_pwd' : 'userPassword',
                    'ldap_user_uid_number' : 'userUidNumber',
                    'ldap_user_gid_number' : 'userGidNumber',
                    'ldap_user_gecos' : 'userGecos',
                    'ldap_user_home_directory' : 'userHomeDirectory',
                    'ldap_user_shell' : 'userShell',
                    'ldap_user_principal' : 'userPrincipal',
                    'ldap_force_upper_case_realm' : 'force_upper_case_realm',
                    'ldap_user_fullname' : 'userFullname',
                    'ldap_user_member_of' : 'userMemberOf',
                    'ldap_user_modify_timestamp' : 'modifyTimestamp',
                    'ldap_group_search_base' : 'groupSearchBase',
                    'ldap_group_search_scope' : 'groupSearchScope',
                    'ldap_group_search_filter' : 'groupSearchFilter',
                    'ldap_group_object_class' : 'groupObjectClass',
                    'ldap_group_name' : 'groupName',
                    'ldap_group_pwd' : 'userPassword',
                    'ldap_group_gid_number' : 'groupGidNumber',
                    'ldap_group_member' : 'groupMember',
                    'ldap_group_modify_timestamp' : 'modifyTimestamp',
                    'ldap_network_timeout' : 'network_timeout',
                    'ldap_offline_timeout' : 'offline_timeout',
                    'ldap_enumeration_refresh_timeout' : 'enumeration_refresh_timeout',
                    'ldap_stale_time' : 'stale_time',
                    'ldap_opt_timeout' : 'opt_timeout',
                    'ldap_tls_reqcert' : 'tls_reqcert',
                    'ldap_netgroup_search_base' : 'netgroupSearchBase',
                    'ldap_netgroup_object_class' : 'netgroupObjectClass',
                    'ldap_netgroup_name' : 'netgroupName',
                    'ldap_netgroup_member' : 'netgroupMember',
                    'ldap_netgroup_triple' : 'netgroupTriple',
                    'ldap_netgroup_modify_timestamp' : 'netgroupModifyTimestamp',
                   }
        krb5_kw = { 'krb5_server' : 'krb5KDCIP',
                    'krb5_realm'  : 'krb5REALM',
                    'krb5_try_simple_upn' : 'krb5try_simple_upn',
                    'krb5_changepw_principal' : 'krb5changepw_principle',
                    'krb5_ccachedir' : 'krb5ccache_dir',
                    'krb5_auth_timeout' : 'krb5auth_timeout',
                    'krb5_ccname_template' : 'krb5ccname_template',
                  }
        user_defaults_kw = { 'default_shell' : 'defaultShell',
                             'base_directory' : 'baseDirectory',
                           }

        self._migrate_enumerate(domain['value'])
        self.rename_opts(domain['name'], generic_kw)
        self.rename_opts(domain['name'], proxy_kw)
        self.rename_opts(domain['name'], ldap_kw)
        self.rename_opts(domain['name'], krb5_kw)

        # remove obsolete libPath option
        self.delete_option_subtree(domain['value'], 'option', 'libPath')

        # configuration files before 0.5.0 did not enforce provider= in local domains
        # it did special-case by domain name (LOCAL)
        prvindex, prv = self.findOpts(domain['value'], 'option', 'id_provider')
        if not prv and domain['name'] == 'domain/LOCAL':
            prv = { 'type'  : 'option',
                    'name'  : 'id_provider',
                    'value' : 'local',
                  }
            domain['value'].insert(0, prv)

        # if domain was local, update with parameters from [user_defaults]
        if prv['value'] == 'local':
            self._update_option(domain['name'], 'user_defaults', user_defaults_kw.values())
            self.delete_option('section', 'user_defaults')
            self.rename_opts(domain['name'], user_defaults_kw)

        # if domain had provider = files, unroll that into provider=proxy, proxy_lib_name=files
        if prv['value'] == 'files':
            prv['value'] = 'proxy'
            libkw = { 'type'  : 'option',
                      'name'  : 'proxy_lib_name',
                      'value' : 'files',
                    }
            domain['value'].insert(prvindex+1, libkw)

    def _migrate_domains(self):
        for domain in [ s for s in self.sections() if s['name'].startswith("domains/") ]:
            self._migrate_domain(domain)

    def _update_if_exists(self, opt, to_name, from_section, from_name):
        index, item = self.get_option_index(from_section, from_name)
        if item:
            item['name'] = to_name
            opt.append(item)

    def _migrate_services(self):
        # [service] - options common to all services, no section as in v1
        service_kw = { 'reconnection_retries' : 'reconnection_retries',
                       'debug_level' : 'debug-level',
                       'debug_timestamps' : 'debug-timestamps',
                       'command' : 'command',
                       'timeout' : 'timeout',
                     }

        # rename services sections
        names_kw = { 'nss' : 'services/nss',
                     'pam' : 'services/pam',
                     'dp'  : 'services/dp',
                   }
        self.rename_opts(None, names_kw, 'section')

        # [sssd] - monitor service
        sssd_kw = [
                    { 'type'  : 'option',
                      'name'  : 'config_file_version',
                      'value' : '2',
                      'action': 'set',
                    }
                  ]
        self._update_if_exists(sssd_kw, 'domains',
                               'domains', 'domains')
        self._update_if_exists(sssd_kw, 'services',
                               'services', 'activeServices')
        self._update_if_exists(sssd_kw, 'sbus_timeout',
                               'services/monitor', 'sbusTimeout')
        self._update_if_exists(sssd_kw, 're_expression',
                              'names', 're-expression')
        self._update_if_exists(sssd_kw, 're_expression',
                              'names', 'full-name-format')
        self.add_section('sssd', sssd_kw)
        # update from general services section and monitor
        self._update_option('sssd', 'services', service_kw.values())
        self._update_option('sssd', 'services/monitor', service_kw.values())

        # [nss] - Name service
        nss_kw = { 'enum_cache_timeout' : 'EnumCacheTimeout',
                   'entry_cache_timeout' : 'EntryCacheTimeout',
                   'entry_cache_nowait_timeout' : 'EntryCacheNoWaitRefreshTimeout',
                   'entry_negative_timeout ' : 'EntryNegativeTimeout',
                   'filter_users' : 'filterUsers',
                   'filter_groups' : 'filterGroups',
                   'filter_users_in_groups' : 'filterUsersInGroups',
                   }
        nss_kw.update(service_kw)
        self._update_option('nss', 'services', service_kw.values())
        self.rename_opts('nss', nss_kw)

        # [pam] - Authentication service
        pam_kw = {}
        pam_kw.update(service_kw)
        self._update_option('pam', 'services', service_kw.values())
        self.rename_opts('pam', pam_kw)

        # remove obsolete sections
        self.delete_option('section', 'services')
        self.delete_option('section', 'names')
        self.delete_option('section', 'domains')
        self.delete_option('section', 'services/monitor')

    def v2_changes(self, out_file_name, backup=True):
        # read in the old file, make backup if needed
        if backup:
            self._backup_file(self.filename)

        self._do_v2_changes()

        # all done, write the file
        of = open(out_file_name, "wb")
        output = self.dump(self.opts)
        of.write(output)
        of.close()
        # make sure it has the right permissions too
        os.chmod(out_file_name, 0600)

    def upgrade_v2(self, out_file_name, backup=True):
        # read in the old file, make backup if needed
        if backup:
            self._backup_file(self.filename)

        # do the migration to v2 format
        # do the upgrade
        self._migrate_services()
        self._migrate_domains()
        # also include any changes in the v2 format
        self._do_v2_changes()

        # all done, write the file
        of = open(out_file_name, "wb")
        output = self.dump(self.opts)
        of.write(output)
        of.close()
        # make sure it has the right permissions too
        os.chmod(out_file_name, 0600)

def parse_options():
    parser = OptionParser()
    parser.add_option("-f", "--file",
                      dest="filename", default="/etc/sssd/sssd.conf",
                      help="Set input file to FILE", metavar="FILE")
    parser.add_option("-o", "--outfile",
                      dest="outfile", default=None,
                      help="Set output file to OUTFILE", metavar="OUTFILE")
    parser.add_option("", "--no-backup", action="store_false",
                      dest="backup", default=True,
                      help="""Do not provide backup file after conversion.
The script copies the original file with the suffix .bak
by default""")
    parser.add_option("-v", "--verbose", action="store_true",
                      dest="verbose", default=False,
                      help="Be verbose")
    (options, args) = parser.parse_args()
    if len(args) > 0:
        print >>sys.stderr, "Stray arguments: %s" % ' '.join([a for a in args])
        return None

    # do the conversion in place by default
    if not options.outfile:
        options.outfile = options.filename

    return options

def verbose(msg, verbose):
    if verbose:
        print msg

def main():
    options = parse_options()
    if not options:
        print >>sys.stderr, "Cannot parse options"
        return 1

    try:
        config = SSSDConfigFile(options.filename)
    except SyntaxError:
        verbose(traceback.format_exc(), options.verbose)
        print >>sys.stderr, "Cannot parse config file %s" % options.filename
        return 1
    except Exception, e:
        print "ERROR: %s" % e
        verbose(traceback.format_exc(), options.verbose)
        return 1

    # make sure we keep strict settings when creating new files
    os.umask(0077)

    version = config.get_version()
    if version == 2:
        verbose("Looks like v2, only checking changes", options.verbose)
        try:
            config.v2_changes(options.outfile, options.backup)
        except Exception, e:
            print "ERROR: %s" % e
            verbose(traceback.format_exc(), options.verbose)
            return 1
    elif version == 1:
        verbose("Looks like v1, performing full upgrade", options.verbose)
        try:
            config.upgrade_v2(options.outfile, options.backup)
        except Exception, e:
            print "ERROR: %s" % e
            verbose(traceback.format_exc(), options.verbose)
            return 1
    else:
        print >>sys.stderr, "Can only upgrade from v1 to v2, file %s looks like version %d" % (options.filename, config.get_version())
        return 1

    return 0

if __name__ == "__main__":
    ret = main()
    sys.exit(ret)




#!/usr/bin/python
#  SSSD
#
#  Unit tests for pysss_murmur
#
#  Copyright (C) Sumit Bose <sbose@redhat.com>        2012
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
import sys
import os
import copy

srcdir = os.getenv('builddir')
if not srcdir:
    srcdir = "."
MODPATH = srcdir + "/.libs" #FIXME - is there a way to get this from libtool?

def compat_assertItemsEqual(this, expected_seq, actual_seq, msg=None):
    return this.assertEqual(sorted(expected_seq), sorted(actual_seq))

def compat_assertIsInstance(this, obj, cls, msg=None):
    return this.assertTrue(isinstance(obj, cls))

# add compat methods for old unittest.TestCase versions
# (python < 2.7, RHEL5 for instance)
if not hasattr(unittest.TestCase, "assertItemsEqual"):
    setattr(unittest.TestCase, "assertItemsEqual", compat_assertItemsEqual)
if not hasattr(unittest.TestCase, "assertIsInstance"):
    setattr(unittest.TestCase, "assertIsInstance", compat_assertIsInstance)

class PySssMurmurImport(unittest.TestCase):
    def setUp(self):
        " Make sure we load the in-tree module "
        self.system_path = sys.path[:]
        sys.path = [ MODPATH ]

    def tearDown(self):
        " Restore the system path "
        sys.path = self.system_path

    def testImport(self):
        " Import the module and assert it comes from tree "
        try:
            import pysss_murmur
        except ImportError, e:
            print >>sys.stderr, "Could not load the pysss_murmur module. Please check if it is compiled"
            raise e
        self.assertEqual(pysss_murmur.__file__, MODPATH + "/pysss_murmur.so")

class PySssMurmurTest(unittest.TestCase):
    def testExpectedHash(self):
        hash = pysss_murmur.murmurhash3("S-1-5-21-2153326666-2176343378-3404031434", 41, 0xdeadbeef)
        self.assertEqual(hash, 93103853)

    def testInvalidArguments(self):
        self.assertRaises(ValueError, pysss_murmur.murmurhash3, 1, 2, 3)
        self.assertRaises(ValueError, pysss_murmur.murmurhash3, "test", 2)
        self.assertRaises(ValueError, pysss_murmur.murmurhash3, "test")
        self.assertRaises(ValueError, pysss_murmur.murmurhash3)
        self.assertRaises(ValueError, pysss_murmur.murmurhash3, "test", -1, 3)
        self.assertRaises(ValueError, pysss_murmur.murmurhash3, "test", 2,
                          0xffffffffff)
        self.assertRaises(ValueError, pysss_murmur.murmurhash3, "test",
                          0xffffffffff, 3)


if __name__ == "__main__":
    error = 0

    suite = unittest.TestLoader().loadTestsFromTestCase(PySssMurmurImport)
    res = unittest.TextTestRunner().run(suite)
    if not res.wasSuccessful():
        error |= 0x1
        # need to bail out here because pysss_murmur could not be imported
        sys.exit(error)

    # import the pysss_murmur module into the global namespace, but make sure
    # it's the one in tree
    sys.path.insert(0, MODPATH)
    import pysss_murmur

    suite = unittest.TestLoader().loadTestsFromTestCase(PySssMurmurTest)
    res = unittest.TextTestRunner().run(suite)
    if not res.wasSuccessful():
        error |= 0x2

    sys.exit(error)



#!/usr/bin/python

import unittest
import sys
import os
import copy

srcdir = os.getenv('builddir')
if not srcdir:
    srcdir = "."
MODPATH = srcdir + "/.libs" #FIXME - is there a way to get this from libtool?

def compat_assertItemsEqual(this, expected_seq, actual_seq, msg=None):
    return this.assertEqual(sorted(expected_seq), sorted(actual_seq))

def compat_assertIsInstance(this, obj, cls, msg=None):
    return this.assertTrue(isinstance(obj, cls))

# add compat methods for old unittest.TestCase versions
# (python < 2.7, RHEL5 for instance)
if not hasattr(unittest.TestCase, "assertItemsEqual"):
    setattr(unittest.TestCase, "assertItemsEqual", compat_assertItemsEqual)
if not hasattr(unittest.TestCase, "assertIsInstance"):
    setattr(unittest.TestCase, "assertIsInstance", compat_assertIsInstance)

class PyHbacImport(unittest.TestCase):
    def setUp(self):
        " Make sure we load the in-tree module "
        self.system_path = sys.path[:]
        sys.path = [ MODPATH ]

    def tearDown(self):
        " Restore the system path "
        sys.path = self.system_path

    def testImport(self):
        " Import the module and assert it comes from tree "
        try:
            import pyhbac
        except ImportError, e:
            print >>sys.stderr, "Could not load the pyhbac module. Please check if it is compiled"
            raise e
        self.assertEqual(pyhbac.__file__, MODPATH + "/pyhbac.so")

class PyHbacRuleElementTest(unittest.TestCase):
    def testInstantiateEmpty(self):
        el = pyhbac.HbacRuleElement()
        self.assertItemsEqual(el.names, [])
        self.assertItemsEqual(el.groups, [])
        self.assertItemsEqual(el.category, set([pyhbac.HBAC_CATEGORY_NULL]))

    def testInit(self):
        names = [ "foo", "bar" ]
        el = pyhbac.HbacRuleElement(names=names)
        self.assertItemsEqual(el.names, names)

        groups = [ "abc", "def" ]
        el = pyhbac.HbacRuleElement(groups=groups)
        self.assertItemsEqual(el.groups, groups)

    def testGetSet(self):
        names = [ "foo", "bar" ]
        el = pyhbac.HbacRuleElement()
        self.assertItemsEqual(el.names, [])
        el.names = names
        self.assertItemsEqual(el.names, names)

        groups = [ "abc", "def" ]
        el = pyhbac.HbacRuleElement()
        self.assertItemsEqual(el.groups, [])
        el.groups = groups
        self.assertItemsEqual(el.groups, groups)

        # Test other iterables than list
        groups = ( "abc", "def" )
        el = pyhbac.HbacRuleElement()
        self.assertItemsEqual(el.groups, [])
        el.groups = groups
        self.assertItemsEqual(el.groups, groups)

    def testCategory(self):
        el = pyhbac.HbacRuleElement()
        assert pyhbac.HBAC_CATEGORY_NULL in el.category
        assert pyhbac.HBAC_CATEGORY_ALL not in el.category

        el.category.add(pyhbac.HBAC_CATEGORY_ALL)
        assert pyhbac.HBAC_CATEGORY_ALL in el.category

        el.category = set([pyhbac.HBAC_CATEGORY_ALL])
        assert pyhbac.HBAC_CATEGORY_ALL in el.category

        # negative tests
        self.assertRaises(TypeError, el.__setattr__, "category", [pyhbac.HBAC_CATEGORY_ALL])
        self.assertRaises(TypeError, el.__setattr__, "category", None)
        self.assertRaises(TypeError, el.__setattr__, "category", 1)

    def testNotIterable(self):
        self.assertRaises(TypeError, pyhbac.HbacRuleElement, names=123)
        self.assertRaises(TypeError, pyhbac.HbacRuleElement, names=None)

    def testRuleElementReference(self):
        def _get_rule():
            users = [ "foo", "bar" ]
            user_groups = [ "abc", "def" ]
            return pyhbac.HbacRuleElement(names=users, groups=user_groups)

        el = _get_rule()
        self.assertItemsEqual(el.names, [ "foo", "bar" ])
        self.assertItemsEqual(el.groups, [ "abc", "def" ])

    def testRepr(self):
        el = pyhbac.HbacRuleElement()
        self.assertEquals(el.__repr__(), u'<category 0 names [] groups []>')

        el.category.add(pyhbac.HBAC_CATEGORY_ALL)
        el.names = ['foo']
        el.groups = ['bar, baz']
        self.assertEquals(el.__repr__(), u'<category 1 names [foo] groups [bar, baz]>')
        

class PyHbacRuleTest(unittest.TestCase):
    def testRuleGetSetName(self):
        name = "testGetRule"
        new_name = "testGetNewRule"

        rule = pyhbac.HbacRule(name)
        self.assertEqual(rule.name, unicode(name))

        rule.name = new_name
        self.assertEqual(rule.name, unicode(new_name))

    def testRuleGetSetEnabled(self):
        rule = pyhbac.HbacRule("testRuleGetSetEnabled")

        rule.enabled = True
        self.assertEqual(rule.enabled, True)
        rule.enabled = False
        self.assertEqual(rule.enabled, False)

        rule.enabled = "TRUE"
        self.assertEqual(rule.enabled, True)
        rule.enabled = "FALSE"
        self.assertEqual(rule.enabled, False)

        rule.enabled = "true"
        self.assertEqual(rule.enabled, True)
        rule.enabled = "false"
        self.assertEqual(rule.enabled, False)

        rule.enabled = "True"
        self.assertEqual(rule.enabled, True)
        rule.enabled = "False"
        self.assertEqual(rule.enabled, False)

        rule.enabled = 1
        self.assertEqual(rule.enabled, True)
        rule.enabled = 0
        self.assertEqual(rule.enabled, False)

        # negative test
        self.assertRaises(TypeError, rule.__setattr__, "enabled", None)
        self.assertRaises(TypeError, rule.__setattr__, "enabled", [])
        self.assertRaises(ValueError, rule.__setattr__, "enabled", "foo")
        self.assertRaises(ValueError, rule.__setattr__, "enabled", 5)

    def testRuleElementInRule(self):
        users = [ "foo", "bar" ]
        user_groups = [ "abc", "def" ]

        # rule should contain empty elements after instantiation
        rule = pyhbac.HbacRule("testRuleElement")
        self.assertIsInstance(rule.users, pyhbac.HbacRuleElement)
        self.assertIsInstance(rule.services, pyhbac.HbacRuleElement)
        self.assertIsInstance(rule.targethosts, pyhbac.HbacRuleElement)
        self.assertIsInstance(rule.srchosts, pyhbac.HbacRuleElement)

        self.assertIsInstance(rule.users.names, list)
        self.assertIsInstance(rule.users.groups, list)
        self.assertItemsEqual(rule.users.names, [])
        self.assertItemsEqual(rule.users.groups, [])

        # Assign by copying a HbacRuleElement
        user_el = pyhbac.HbacRuleElement(names=users, groups=user_groups)
        rule = pyhbac.HbacRule("testRuleElement")
        rule.users = user_el
        self.assertItemsEqual(rule.users.names, users)
        self.assertItemsEqual(rule.users.groups, user_groups)

        # Assign directly
        rule = pyhbac.HbacRule("testRuleElement")
        rule.users.names = users
        rule.users.groups = user_groups
        self.assertItemsEqual(rule.users.names, users)
        self.assertItemsEqual(rule.users.groups, user_groups)

    def testRuleElementInRuleReference(self):
        " Test that references to RuleElement are kept even if element goes out of scope "
        def _get_rule():
            users = [ "foo", "bar" ]
            user_groups = [ "abc", "def" ]
            el = pyhbac.HbacRuleElement(names=users, groups=user_groups)
            rule = pyhbac.HbacRule("testRuleElement")
            rule.users = el
            return rule

        rule = _get_rule()
        self.assertItemsEqual(rule.users.names, [ "foo", "bar" ])
        self.assertItemsEqual(rule.users.groups, [ "abc", "def" ])

    def testRepr(self):
        r = pyhbac.HbacRule('foo')
        self.assertEqual(r.__repr__(), u"<name foo enabled 0 "
                                        "users <category 0 names [] groups []> "
                                        "services <category 0 names [] groups []> "
                                        "targethosts <category 0 names [] groups []> "
                                        "srchosts <category 0 names [] groups []>>")

        name = "someuser"
        service = "ssh"
        srchost = "host1"
        targethost = "host2"

        r.users.names = [ name ]
        r.services.names = [ service ]
        r.srchosts.names = [ srchost ]
        r.targethosts.names = [ targethost ]

        self.assertEqual(r.__repr__(), u"<name foo enabled 0 "
                                        "users <category 0 names [%s] groups []> "
                                        "services <category 0 names [%s] groups []> "
                                        "targethosts <category 0 names [%s] groups []> "
                                        "srchosts <category 0 names [%s] groups []>>" %
                                        (name, service, targethost, srchost))

    def testValidate(self):
        r = pyhbac.HbacRule('valid_rule')

        valid, missing = r.validate()
        self.assertEqual(valid, False)
        self.assertItemsEqual(missing, ( pyhbac.HBAC_RULE_ELEMENT_USERS,
                                         pyhbac.HBAC_RULE_ELEMENT_SERVICES,
                                         pyhbac.HBAC_RULE_ELEMENT_TARGETHOSTS,
                                         pyhbac.HBAC_RULE_ELEMENT_SOURCEHOSTS ))

        r.users.names = [ "someuser" ]
        r.services.names = [ "ssh" ]

        valid, missing = r.validate()
        self.assertEqual(valid, False)
        self.assertItemsEqual(missing, ( pyhbac.HBAC_RULE_ELEMENT_TARGETHOSTS,
                                         pyhbac.HBAC_RULE_ELEMENT_SOURCEHOSTS ))

        r.srchosts.names = [ "host1" ]
        r.targethosts.names = [ "host2" ]

        valid, missing = r.validate()
        self.assertEqual(valid, True)

class PyHbacRequestElementTest(unittest.TestCase):
    def testInstantiateEmpty(self):
        el = pyhbac.HbacRequestElement()
        self.assertItemsEqual(el.name, "")
        self.assertItemsEqual(el.groups, [])

    def testInit(self):
        name = "foo"
        el = pyhbac.HbacRequestElement(name=name)
        self.assertItemsEqual(el.name, name)

        groups = [ "abc", "def" ]
        el = pyhbac.HbacRequestElement(groups=groups)
        self.assertItemsEqual(el.groups, groups)

    def testGetSet(self):
        name = "foo"
        el = pyhbac.HbacRequestElement()
        self.assertItemsEqual(el.name, "")
        el.name = name
        self.assertItemsEqual(el.name, name)

        groups = [ "abc", "def" ]
        el = pyhbac.HbacRequestElement()
        self.assertItemsEqual(el.groups, [])
        el.groups = groups
        self.assertItemsEqual(el.groups, groups)

        # Test other iterables than list
        groups = ( "abc", "def" )
        el = pyhbac.HbacRequestElement()
        self.assertItemsEqual(el.groups, [])
        el.groups = groups
        self.assertItemsEqual(el.groups, groups)

    def testGroupsNotIterable(self):
        self.assertRaises(TypeError, pyhbac.HbacRequestElement, groups=None)
        self.assertRaises(TypeError, pyhbac.HbacRequestElement, groups=123)

    def testRepr(self):
        r = pyhbac.HbacRequestElement()
        self.assertEqual(r.__repr__(), u"<name  groups []>")

        r.name = 'foo'
        r.groups = ['bar', 'baz']
        self.assertEqual(r.__repr__(), u"<name foo groups [bar,baz]>")

class PyHbacRequestTest(unittest.TestCase):
    def testRequestElementHandling(self):
        name = "req_name"
        groups = [ "g1", "g2" ]

        # The request should be empty after instantiation
        req = pyhbac.HbacRequest()
        self.assertIsInstance(req.user, pyhbac.HbacRequestElement)
        self.assertIsInstance(req.service, pyhbac.HbacRequestElement)
        self.assertIsInstance(req.targethost, pyhbac.HbacRequestElement)
        self.assertIsInstance(req.srchost, pyhbac.HbacRequestElement)

        self.assertEqual(req.user.name, "")
        self.assertIsInstance(req.user.groups, list)
        self.assertItemsEqual(req.user.groups, [])

        # Assign by copying a HbacRequestElement
        user_el = pyhbac.HbacRequestElement(name=name, groups=groups)
        req = pyhbac.HbacRequest()
        req.user = user_el
        self.assertItemsEqual(req.user.name, name)
        self.assertItemsEqual(req.user.groups, groups)

        # Assign directly
        req = pyhbac.HbacRequest()
        req.user.name = name
        req.user.groups = groups
        self.assertItemsEqual(req.user.name, name)
        self.assertItemsEqual(req.user.groups, groups)

    def testRuleName(self):
        req = pyhbac.HbacRequest()
        self.assertEqual(req.rule_name, None)
        # python 2.4 raises TypError, 2.7 raises AttributeError
        self.assertRaises((TypeError, AttributeError), req.__setattr__, "rule_name", "foo")

    def testEvaluate(self):
        name = "someuser"
        service = "ssh"
        srchost = "host1"
        targethost = "host2"

        allow_rule = pyhbac.HbacRule("allowRule", enabled=True)
        allow_rule.users.names = [ name ]
        allow_rule.services.names = [ service ]
        allow_rule.srchosts.names = [ srchost ]
        allow_rule.targethosts.names = [ targethost ]

        req = pyhbac.HbacRequest()
        req.user.name = name
        req.service.name = service
        req.srchost.name = srchost
        req.targethost.name = targethost

        # Test that an allow rule on its own allows access
        res = req.evaluate((allow_rule,))
        self.assertEqual(res, pyhbac.HBAC_EVAL_ALLOW)
        self.assertEqual(req.rule_name, "allowRule")

        # Test that a user not in the rule is not allowed
        savename = req.user.name
        req.user.name = "someotheruser"
        res = req.evaluate((allow_rule, ))
        self.assertEqual(res, pyhbac.HBAC_EVAL_DENY)
        self.assertEqual(req.rule_name, None)

        # But allows if the rule is an ALL rule
        allow_rule.users.category.add(pyhbac.HBAC_CATEGORY_ALL)
        res = req.evaluate((allow_rule, ))
        self.assertEqual(res, pyhbac.HBAC_EVAL_ALLOW)

    def testRepr(self):
        name = "someuser"
        service = "ssh"
        srchost = "host1"
        targethost = "host2"

        req = pyhbac.HbacRequest()

        self.assertEqual(req.__repr__(), "<user <name  groups []> "
                                         "service <name  groups []> "
                                         "targethost <name  groups []> "
                                         "srchost <name  groups []>>")

        req.user.name = name
        req.service.name = service
        req.srchost.name = srchost
        req.targethost.name = targethost

        self.assertEqual(req.__repr__(), "<user <name %s groups []> "
                                         "service <name %s groups []> "
                                         "targethost <name %s groups []> "
                                         "srchost <name %s groups []>>" %
                                         (name, service, targethost, srchost))

    def testEvaluateNegative(self):
        name = "someuser"
        service = "ssh"
        srchost = "host1"
        targethost = "host2"

        allow_rule = pyhbac.HbacRule("allowRule", enabled=True)
        allow_rule.users.names = [ name ]
        allow_rule.services.names = [ service ]
        allow_rule.srchosts.names = [ srchost ]
        allow_rule.targethosts.names = [ targethost ]

        req = pyhbac.HbacRequest()
        req.service.name = service
        req.srchost.name = srchost
        req.targethost.name = targethost
        req.user.name = name

        saveuser = req.user
        req.user = None # need to catch this

        # catch invalid category value
        savecat = copy.copy(allow_rule.users.category)
        allow_rule.users.category.add(pyhbac.HBAC_EVAL_ERROR)
        self.assertRaises(ValueError, req.evaluate, (allow_rule,))
        allow_rule.users.category = savecat

        # Test that invalid type is raised
        self.assertRaises(TypeError, req.evaluate, (allow_rule,))

        req.user = saveuser
        allow_rule.users = None # need to catch this
        self.assertRaises(TypeError, req.evaluate, (allow_rule,))

        # catch invalid rule type
        self.assertRaises(TypeError, req.evaluate, (allow_rule, None))

class PyHbacModuleTest(unittest.TestCase):
    def testHasResultTypes(self):
        assert hasattr(pyhbac, "HBAC_EVAL_ALLOW")
        assert hasattr(pyhbac, "HBAC_EVAL_DENY")
        assert hasattr(pyhbac, "HBAC_EVAL_ERROR")

    def testHasErrorTypes(self):
        assert hasattr(pyhbac, "HBAC_ERROR_UNKNOWN")
        assert hasattr(pyhbac, "HBAC_SUCCESS")
        assert hasattr(pyhbac, "HBAC_ERROR_NOT_IMPLEMENTED")
        assert hasattr(pyhbac, "HBAC_ERROR_OUT_OF_MEMORY")
        assert hasattr(pyhbac, "HBAC_ERROR_UNPARSEABLE_RULE")

    def testHasCategories(self):
        assert hasattr(pyhbac, "HBAC_CATEGORY_NULL")
        assert hasattr(pyhbac, "HBAC_CATEGORY_ALL")

    def testHasRuleElementTypes(self):
        assert hasattr(pyhbac, "HBAC_RULE_ELEMENT_USERS")
        assert hasattr(pyhbac, "HBAC_RULE_ELEMENT_SERVICES")
        assert hasattr(pyhbac, "HBAC_RULE_ELEMENT_TARGETHOSTS")
        assert hasattr(pyhbac, "HBAC_RULE_ELEMENT_SOURCEHOSTS")

    def testHbacResultString(self):
        results = [ pyhbac.HBAC_EVAL_ALLOW, pyhbac.HBAC_EVAL_DENY,
                    pyhbac.HBAC_EVAL_ERROR ]
        for r in results:
            s = pyhbac.hbac_result_string(r)
            self.assertIsInstance(s, unicode)
            assert len(s) > 0

    def testHbacErrorString(self):
        errors = [ pyhbac.HBAC_ERROR_UNKNOWN,
                   pyhbac.HBAC_SUCCESS,
                   pyhbac.HBAC_ERROR_NOT_IMPLEMENTED,
                   pyhbac.HBAC_ERROR_OUT_OF_MEMORY,
                   pyhbac.HBAC_ERROR_UNPARSEABLE_RULE ]
        for e in errors:
            s = pyhbac.hbac_error_string(e)
            self.assertIsInstance(s, unicode)
            assert len(s) > 0


if __name__ == "__main__":
    error = 0

    suite = unittest.TestLoader().loadTestsFromTestCase(PyHbacImport)
    res = unittest.TextTestRunner().run(suite)
    if not res.wasSuccessful():
        error |= 0x1
        # need to bail out here because pyhbac could not be imported
        sys.exit(error)

    # import the pyhbac module into the global namespace, but make sure it's
    # the one in tree
    sys.path.insert(0, MODPATH)
    import pyhbac

    suite = unittest.TestLoader().loadTestsFromTestCase(PyHbacRuleElementTest)
    res = unittest.TextTestRunner().run(suite)
    if not res.wasSuccessful():
        error |= 0x2

    suite = unittest.TestLoader().loadTestsFromTestCase(PyHbacRuleTest)
    res = unittest.TextTestRunner().run(suite)
    if not res.wasSuccessful():
        error |= 0x3

    suite = unittest.TestLoader().loadTestsFromTestCase(PyHbacRequestElementTest)
    res = unittest.TextTestRunner().run(suite)
    if not res.wasSuccessful():
        error |= 0x4

    suite = unittest.TestLoader().loadTestsFromTestCase(PyHbacRequestTest)
    res = unittest.TextTestRunner().run(suite)
    if not res.wasSuccessful():
        error |= 0x5

    suite = unittest.TestLoader().loadTestsFromTestCase(PyHbacModuleTest)
    res = unittest.TextTestRunner().run(suite)
    if not res.wasSuccessful():
        error |= 0x6

    sys.exit(error)

